SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CardTest methodsFor: 'expiration' stamp: 'JuanEdi 6/8/2013 11:25'!
testShouldBeExpiredIfCurrentMonthIsAfterExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 3 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth).! !

!CardTest methodsFor: 'expiration' stamp: 'JuanEdi 6/8/2013 11:26'!
testShouldNotBeExpiredIfCurrentMonthIsBeforeExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 1 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth) not.! !

!CardTest methodsFor: 'expiration' stamp: 'JuanEdi 6/8/2013 11:26'!
testShouldNotBeExpiredIfCurrentMonthIsExpirationMonth
	| aCard expirationMonth currentMonth |
	
	expirationMonth := Month month: 2 year: 2000.
	currentMonth := Month month: 2 year: 2000.
	
	aCard := Card ownedBy: 'Luis' withNumber: 123 expiring: expirationMonth.
	
	self assert: (aCard isExpiredWithCurrentMonthBeing: currentMonth) not.! !

TestCase subclass: #CashierTest
	instanceVariableNames: 'catalog calendar expiredCard validCard paymentTerminalStub cashier transactionCount lastCardUsed lastAmountProcessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CashierTest methodsFor: 'running' stamp: 'pb 6/9/2013 11:54'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.
	
	calendar := FixedCalendar setIn: (Date newDay: 1 month: 2 year: 2000 ).
	
	expiredCard := Card ownedBy: 'Marcelo' withNumber: 123 expiring: (Month month: 1 year: 2000).
	validCard := Card ownedBy: 'Marcelo' withNumber: 456 expiring: (Month month: 3 year: 2000).
	
	paymentTerminalStub := PluggableStubPaymentTerminal new.
	transactionCount := 0.
	
	cashier := Cashier usingCatalog: catalog paymentTerminal: paymentTerminalStub andCalendar: calendar.
! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 01:02'!
testShouldDebitTheCartAmountInTheTerminalWhenCheckingOutACart
	| aCart expectedAmount |
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	paymentTerminalStub
		onDebitDo: [ :anAmount :aCard | 
			transactionCount := transactionCount + 1.
			lastCardUsed := aCard.
			lastAmountProcessed := anAmount ].
	cashier checkout: aCart forCustomer: 'Pedro' usingCard: validCard.
	expectedAmount := 2 * (catalog at: 'Harry Potter') + (3 * catalog at: 'Dracula').
	self assert: lastCardUsed = validCard.
	self assert: lastAmountProcessed = expectedAmount! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 01:03'!
testShouldFailIfTryingToCheckoutEmptyCart
	| initialTransactionCount registeredSalesSize |
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1 ].
	registeredSalesSize := cashier salesBook size.
	self
		should: [ cashier checkout: (ShoppingCart newWithCatalog: catalog) forCustomer: 'Pedro' usingCard: validCard ]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier emptyCartCheckoutMessage.
			self assert: transactionCount = initialTransactionCount.
			self assert: cashier salesBook size = registeredSalesSize ]! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 01:04'!
testShouldFailIfTryingToCheckoutWithExpiredCreditCard
	| aCart initialTransactionCount registeredSalesSize |
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1 ].
	registeredSalesSize := cashier salesBook size.
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	self
		should: [ cashier checkout: aCart forCustomer: 'Pedro' usingCard: expiredCard ]
		raise: Error
		withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier expiredCardCheckoutMessage.
			self assert: transactionCount = initialTransactionCount.
			self assert: cashier salesBook size = registeredSalesSize ]! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 01:04'!
testShouldFailIfTryingToUseCardWithNoCredit
	| aCashier aPaymentTerminal aCart registeredSalesSize |
	aPaymentTerminal := PluggableStubPaymentTerminal new.
	aPaymentTerminal onDebitDo: [ :anAmount :aCard | CardWithNoCreditError signal ].
	aCashier := Cashier usingCatalog: catalog paymentTerminal: aPaymentTerminal andCalendar: calendar.
	registeredSalesSize := aCashier salesBook size.
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	self
		should: [ aCashier checkout: aCart forCustomer: 'Pedro' usingCard: validCard ]
		raise: CardWithNoCreditError
		withExceptionDo: [ :anException | self assert: aCashier salesBook size = registeredSalesSize ]! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 02:19'!
testShouldPrintTicketWhenCheckingOutAValidCart
	| aCart ticket expectedAmount |
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	ticket := cashier checkout: aCart forCustomer: 'Pedro' usingCard: validCard.
	expectedAmount := 2 * (catalog at: 'Harry Potter') + (3 * catalog at: 'Dracula').
	self assert: ticket totalAmount = expectedAmount! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 01:04'!
testShouldProcessAndRegisterTransactionWhenCheckingOutValidCart
	| aCart initialTransactionCount salesCount |
	initialTransactionCount := transactionCount.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1 ].
	salesCount := cashier salesBook size.
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	cashier checkout: aCart forCustomer: 'Pedro' usingCard: validCard.
	self assert: transactionCount = (initialTransactionCount + 1).
	self assert: cashier salesBook size = (salesCount + 1)! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 02:19'!
testShouldRegisterTheSaleWithTheRightAmountWhenCheckingOutACart
	| aCart expectedAmount lastSale |
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOccurrences: 2.
	aCart add: 'Dracula' withOccurrences: 3.
	paymentTerminalStub onDebitDo: [ :anAmount :aCard | transactionCount := transactionCount + 1 ].
	cashier checkout: aCart forCustomer: 'Pedro' usingCard: validCard.
	expectedAmount := 2 * (catalog at: 'Harry Potter') + (3 * catalog at: 'Dracula').
	lastSale := cashier salesBook last.
	self assert: lastSale totalAmount = expectedAmount! !

TestCase subclass: #ConsecutiveNumberSequenceTest
	instanceVariableNames: 'nextValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ConsecutiveNumberSequenceTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 23:33'!
testShouldSIncrementValuesByOne

	| sequence |
	
	sequence := ConsecutiveNumberSequence new.
	(1 to: 10) do: [ :expectedValue | self assert: expectedValue = sequence nextValue ]! !

!ConsecutiveNumberSequenceTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 23:31'!
testShouldStartAtOne

	| sequence |
	sequence := ConsecutiveNumberSequence new.
	self assert: 1 = sequence nextValue.! !

TestCase subclass: #RestInterfaceTest
	instanceVariableNames: 'userRegistry catalog restInterface bookRegistry clock cartsSessions cashier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RestInterfaceTest methodsFor: 'running' stamp: 'JuanEdi 6/9/2013 23:49'!
setUp
	
	| calendar paymentTerminalStub |
	
	cartsSessions := Dictionary new.
	userRegistry := Dictionary new.
	bookRegistry := Dictionary new.
	catalog := Dictionary new.
	
	clock := FixedClock new.
	clock fixTimeAt: 12 hours. 
	
	userRegistry at: 1 put: '1111'.
	userRegistry at: 2 put: '2222'.
	
	catalog at: 'ISBN1' put: 10.
	catalog at: 'ISBN2' put: 20.
	
	calendar := FixedCalendar setIn: (Date newDay: 1 month: 2 year: 2000 ).
	paymentTerminalStub := PluggableStubPaymentTerminal new.
	cashier := Cashier usingCatalog: catalog paymentTerminal: paymentTerminalStub andCalendar: calendar.
	
	restInterface := RestInterface newUsingCartSessions: cartsSessions
										userRegistry: userRegistry
										catalog: catalog
										cashier: cashier
										andClock: clock.! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'pb 6/9/2013 15:52'!
testCreatesEmptyCartAndReturnsTheIdAfterCreation

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self assert: cartsSessions size = 1.
	self assert: (cartsSessions keys includes: cartId).
	self assert: (cartsSessions at: cartId) cart isEmpty.! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'pb 6/9/2013 18:06'!
testShouldAddElementsToCartWhenUsingAddToCart
	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := (cartsSessions at: cartId) cart.
	
	restInterface addToCart: cartId isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111'.
 
	self assert: cart itemCount = 2.
	self assert: (cart occurrencesOf: 'ISBN1') = 2.	! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'JuanEdi 6/9/2013 22:46'!
testShouldFailIfTryingToAddItemsToACartAfter30MinutesOfInactivity

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	clock add: 41 minutes.
	
	self should: [ restInterface addToCart: cartId isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111' ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownCartMessage.
			self assert: cartsSessions size = 0
		].! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'pb 6/9/2013 18:07'!
testShouldFailIfTryingToAddItemsToInexistentCart

	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := (cartsSessions at: cartId) cart.
	
	self should: [ restInterface addToCart: (cartId + 1) isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111']
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownCartMessage.
			self assert: cartsSessions size = 1.
			self assert: cart isEmpty.
		].! !

!RestInterfaceTest methodsFor: 'checkout cart' stamp: 'JuanEdi 6/9/2013 23:05'!
testShouldFailIfTryingToCheckoutAnInexistentCart
	
	self should: [
		restInterface checkoutCart: 1234
						for: 1
						withPassword: '1111'
						usingCard: 1234
						expiringOn: (Month month: 2 year: 2000)
						andOwner: 'Jose'.
		]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownCartMessage.
		].! !

!RestInterfaceTest methodsFor: 'checkout cart' stamp: 'JuanEdi 6/9/2013 23:47'!
testShouldFailIfTryingToCheckoutEmptyCart

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self should: [ restInterface checkoutCart: cartId for: 1 withPassword: '1111' usingCard: 1234 expiringOn: (Month month: 2 year: 2000) andOwner: 'Jose' ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = Cashier emptyCartCheckoutMessage. 
		]! !

!RestInterfaceTest methodsFor: 'listing carts' stamp: 'FW 6/9/2013 17:44'!
testShouldFailIfTryingToListAnInexistentCart

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self should: [ restInterface listCartWithId: cartId+1 usingUser: 1 andPassword: '1111'. ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownCartMessage.
		].! !

!RestInterfaceTest methodsFor: 'add items' stamp: 'pb 6/9/2013 18:07'!
testShouldFailTryingToAddInexistentBookToCarts

	| cartId cart |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cart := (cartsSessions at: cartId) cart.
	
	self should: [ restInterface addToCart: cartId isbn: 'ISBN40' copies: 2 withUser: 1 andPassword: '1111' ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = RestInterface unknownBookMessage.
			self assert: cartsSessions size = 1.
			self assert: cart isEmpty.
		].! !

!RestInterfaceTest methodsFor: 'authenticating' stamp: 'JuanEdi 6/9/2013 22:57'!
testShouldFailUsingInterfaceWithInvalidPassword
	| newCartBlock messages listCartBlock aCart |
	newCartBlock := [ :user :password | restInterface newCartForUserWithId: user andPassword: password   ].
	listCartBlock := [ :user :password | restInterface listCartWithId: 0 usingUser:  user andPassword: password ].
	
	messages := OrderedCollection with: newCartBlock with: listCartBlock.
	
	aCart := ShoppingCart newWithCatalog: catalog.
		cartsSessions at: 0 put: (Session newForUser: 1 cart: aCart usingClock: clock expiringIn: 10 minutes).
	
	messages do:  [ :message |
		self	 should: [ message value: 1 value: 'wrongPassword']
			raise:  AuthenticationFailed
			withExceptionDo: [ :anError |
				self assert: anError messageText = RestInterface invalidAuthenticationData. 
			]
		]! !

!RestInterfaceTest methodsFor: 'cart creation' stamp: 'pb 6/9/2013 15:54'!
testShouldGenerateNewIdsForEveryCart

	| firstCartId secondCartId thirdCartId |
	
	firstCartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	secondCartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	
	self assert: cartsSessions size = 2.
	self assert: (firstCartId = secondCartId) not.
	
	thirdCartId := restInterface newCartForUserWithId: 2 andPassword: '2222'.
	self assert: (firstCartId = thirdCartId) not.
	self assert: (thirdCartId = secondCartId) not.! !

!RestInterfaceTest methodsFor: 'listing carts' stamp: 'pb 6/9/2013 18:10'!
testShouldListCartAfterAddingElements

	| cartId cartElements |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	restInterface addToCart: cartId isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111'. 
	restInterface addToCart: cartId isbn: 'ISBN2' copies: 1 withUser: 1 andPassword: '1111'. 
	
	cartElements := restInterface listCartWithId: cartId usingUser: 1 andPassword: '1111'.
	
	self assert: cartElements size = 3.
	self assert: (cartElements includes: 'ISBN1').
	self assert: (cartElements includes: 'ISBN2').
	self assert: ((cartElements occurrencesOf: 'ISBN1') = 2).! !

!RestInterfaceTest methodsFor: 'listing carts' stamp: 'pb 6/9/2013 16:55'!
testShouldListEmptyCartReturningAnEmptyList

	| cartId cartElements |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	cartElements := restInterface listCartWithId: cartId usingUser: 1 andPassword: '1111'.
	
	self assert: cartElements isEmpty.! !

!RestInterfaceTest methodsFor: 'checkout cart' stamp: 'JuanEdi 6/9/2013 23:50'!
testShouldPerformSaleAfterCheckingOutAValidCart

	| cartId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	restInterface addToCart: cartId isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111'.
	
	restInterface checkoutCart: cartId for: 1 withPassword: '1111' usingCard: 1234 expiringOn: (Month month: 2 year: 2000) andOwner: 'Jose'.
	self assert: cashier salesBook size = 1.! !

!RestInterfaceTest methodsFor: 'checkout cart' stamp: 'JuanEdi 6/9/2013 23:53'!
testShouldReturnATransactionIdWhenCheckingOutAValidCart
	"El ID de transaccin es algo que se utiliza para retornar al usuario, por eso se valida que se genere un nmero.
	 Dado que no se especifica nada ms que se haga con este nmero, simplemente se genera y se olvida.
	 Eventualmente (cuando sea necesario) se podra guardar en algn ndice para acceder a las transacciones pasadas a travs del ID externo."
	| cartId transactionId |
	
	cartId := restInterface newCartForUserWithId: 1 andPassword: '1111'.
	restInterface addToCart: cartId isbn: 'ISBN1' copies: 2 withUser: 1 andPassword: '1111'.
	
	transactionId := restInterface checkoutCart: cartId for: 1 withPassword: '1111' usingCard: 1234 expiringOn: (Month month: 2 year: 2000) andOwner: 'Jose'.

	self assert: (transactionId isKindOf: Number)! !

TestCase subclass: #SessionTest
	instanceVariableNames: 'cart clock session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!SessionTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:57'!
setUp

	| aCatalog |
	aCatalog := Dictionary new.
	cart := ShoppingCart newWithCatalog: aCatalog.
	clock := FixedClock new.
	clock fixTimeAt: 12 hours.
	
	session := Session newForUser: 123 cart: cart usingClock: clock expiringIn: self timeUntilExpiration.! !

!SessionTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:58'!
testShouldProvideAccessToCartIfItIsExpired

	clock add: self timeUntilExpiration.
	self should: [ session cart ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: (anError isKindOf: SessionExpiredError).
		]! !

!SessionTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:03'!
testShouldProvideAccessToCartIfItIsNotExpired

	self assert: session cart = cart.! !

!SessionTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:23'!
testTimeUntilExpirationShouldBeResetWithEveryUsage

	clock add: self timeUntilExpiration - 1 minute.
	session cart.
	clock add: 1 minute.
	session cart.! !

!SessionTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:12'!
timeUntilExpiration

	^ 10 minutes.! !

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: 'catalog calendar expiredCard validCard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: 'running' stamp: 'JuanEdi 6/2/2013 01:06'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:54'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: catalog) isEmpty.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOccurrences: 2.
	self assert: (cart occurrencesOf: 'Dracula') = 2.
	self assert: (cart itemCount) = 2.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldFailIfTryingToAddANegativeAmountOfProducts

	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter' withOccurrences: 2.
	self should: [ cart add: 'Harry Potter' withOccurrences: -1 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart invalidAmountOfProductsMessage ].
	self assert: (cart occurrencesOf: 'Harry Potter') = 2.
	self assert: (cart itemCount) = 2.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	self should: [ cart add: 'Cien aos de soledad' withOccurrences: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/8/2013 12:23'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOccurrences: 2 .
	cart add: 'Dracula' withOccurrences: 3.
	self assert: (cart occurrencesOf: 'Dracula') = 5.
	self assert: (cart itemCount) = 5.! !

TestCase subclass: #TicketEntryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TicketEntryTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:58'!
testShouldEqualOtherEntryWithSameInformation

	| anEntry otherEntry |
	
	anEntry := TicketEntry withCount: 2 ofItem: '123' withTotalAmount: 15.
	otherEntry := TicketEntry withCount: 2 ofItem: '123' withTotalAmount: 15.
	
	self assert: anEntry = otherEntry.! !

TestCase subclass: #TicketTest
	instanceVariableNames: 'amount anItemBag aCatalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:11'!
setUp
	
	anItemBag := Bag new.
	anItemBag add: 'Harry Potter' withOccurrences: 2.
	anItemBag add: 'Lord of the Rings' withOccurrences: 1.
	
	aCatalog := Dictionary new.
	aCatalog at: 'Harry Potter' put: 5.
	aCatalog at: 'Lord of the Rings' put: 20.! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:25'!
testShouldFailIfAProductIsNotInTheCatalog
	
	| purchases |
	purchases := Bag new.
	purchases add: 'Harry Potter'.
	self should: [ Ticket newForUser: 'Pedro' withPurchases: purchases usingCatalog: Dictionary new ]
			raise: Error
			withExceptionDo: [ :anError | self assert: anError messageText = Ticket unknownProductMessage ]! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:19'!
testShouldFailIfCreatedWithNoPurchases
	
	self should: [ Ticket newForUser: 'Pedro' withPurchases: Bag new usingCatalog: Dictionary new ]
			raise: Error
			withExceptionDo: [ :anError | self assert: anError messageText = Ticket noPurchasesMessage ]! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:12'!
testShouldListAllItemsWithTheirPrices
	
	| ticket expectedEntries |

	ticket := Ticket newForUser: 'Pedro' withPurchases: anItemBag usingCatalog: aCatalog.
	
	expectedEntries := Set new.
	expectedEntries add: (TicketEntry withCount: 2 ofItem: 'Harry Potter' withTotalAmount: 10).
	expectedEntries add: (TicketEntry withCount: 1 ofItem: 'Lord of the Rings' withTotalAmount: 20).
	
	self assert: ticket entries = expectedEntries.! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:22'!
testShouldNotChangeAmountIfCatalogIsUpdated
	
	| ticket |

	ticket := Ticket newForUser: 'Pedro' withPurchases: anItemBag usingCatalog: aCatalog.
	aCatalog at: 'Harry Potter' put: 1000.
	
	self assert: ticket totalAmount = 30.! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:15'!
testShouldProvideTheCustomerName
	
	| ticket |

	ticket := Ticket newForUser: 'Pedro' withPurchases: anItemBag usingCatalog: aCatalog.
	
	self assert: ticket customer = 'Pedro'.! !

!TicketTest methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:13'!
testShouldProvideTheSumOfAllAmounts
	
	| ticket |

	ticket := Ticket newForUser: 'Pedro' withPurchases: anItemBag usingCatalog: aCatalog.
	
	self assert: ticket totalAmount = 30.! !

Object subclass: #Calendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Calendar subclass: #FixedCalendar
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!FixedCalendar commentStamp: '<historical>' prior: 0!
A FixedCalendar is useful when there is a need to fix the date used as the current date during a test.

Instance Variables
	date:		<Date>

date
	- xxxxx
!

!FixedCalendar class methodsFor: 'setup' stamp: 'JuanEdi 6/2/2013 00:38'!
setIn: aDate

	^ self new setIn: aDate.! !

!FixedCalendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:37'!
currentMonth

	^ date month.! !

!FixedCalendar methodsFor: 'initialize-release' stamp: 'JuanEdi 6/2/2013 00:44'!
setIn: aDate

	date := aDate.! !

Calendar subclass: #RealTimeCalendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RealTimeCalendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:55'!
currentMonth

	^ Date today month.! !

Object subclass: #Card
	instanceVariableNames: 'number expirationMonth ownerName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Card class methodsFor: 'instance-creation' stamp: 'JuanEdi 6/8/2013 11:17'!
ownedBy: ownerName withNumber: aCardNumber expiring: aMonth 
	
	^ self new initializeWithOwner: ownerName number: aCardNumber expiringOn: aMonth.! !

!Card methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:35'!
expirationMonth
	
	^ expirationMonth! !

!Card methodsFor: 'initialize-release' stamp: 'JuanEdi 6/8/2013 11:23'!
initializeWithOwner: aName number: aCardNumber expiringOn: aMonth
	
	ownerName := aName.
	number := aCardNumber.
	expirationMonth := aMonth.! !

!Card methodsFor: 'accessing' stamp: 'JuanEdi 6/8/2013 11:21'!
isExpiredWithCurrentMonthBeing: currentMonth 
	
	^ expirationMonth < currentMonth.! !

Object subclass: #Cashier
	instanceVariableNames: 'paymentTerminal catalog calendar salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:33'!
emptyCartCheckoutMessage
	
	^ 'Cannot checkout empty carts.'! !

!Cashier class methodsFor: 'error-messages' stamp: 'JuanEdi 6/2/2013 00:23'!
expiredCardCheckoutMessage
	
	^ 'Cannot checkout a cart with an expired card'! !

!Cashier class methodsFor: 'instance-creation' stamp: 'JuanEdi 6/2/2013 00:31'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	^ self new usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar.! !

!Cashier methodsFor: 'checkout' stamp: 'JuanEdi 6/10/2013 02:19'!
checkout: aCart forCustomer: aCustomer usingCard: aCard
	| totalAmount ticket |
	self validateCartIsNotEmpty: aCart.
	self validateCardIsNotExpired: aCard.
	
	totalAmount := 0.
	aCart do: [ :anItem | totalAmount := totalAmount + (catalog at: anItem) ].
	paymentTerminal debit: totalAmount from: aCard.
	
	ticket := Ticket newForUser: aCustomer withPurchases: aCart elements usingCatalog: catalog.
	salesBook add: ticket.
	
	^ ticket.! !

!Cashier methodsFor: 'sales' stamp: 'pb 6/2/2013 16:28'!
salesBook
	^ salesBook.! !

!Cashier methodsFor: 'initialize-release' stamp: 'pb 6/2/2013 16:24'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	catalog := aCatalog.
	paymentTerminal := aPaymentTerminal.
	calendar := aCalendar.
	salesBook := OrderedCollection new.! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/8/2013 11:25'!
validateCardIsNotExpired: aCard 
	(aCard isExpiredWithCurrentMonthBeing: calendar currentMonth)
		ifTrue: [self error: Cashier expiredCardCheckoutMessage]! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/2/2013 01:00'!
validateCartIsNotEmpty: aShoppingCart 
	aShoppingCart isEmpty ifTrue: [self error: Cashier emptyCartCheckoutMessage]! !

Object subclass: #Clock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Clock methodsFor: 'as yet unclassified' stamp: 'pb 6/9/2013 16:40'!
time
	^ self subclassResponsibility.! !

Clock subclass: #FixedClock
	instanceVariableNames: 'time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!FixedClock methodsFor: 'adding' stamp: 'pb 6/9/2013 16:27'!
add: aDuration 
	self fixTimeAt: self time + aDuration.! !

!FixedClock methodsFor: 'accessing' stamp: 'pb 6/9/2013 16:21'!
fixTimeAt: aTime
	time := aTime.! !

!FixedClock methodsFor: 'accessing' stamp: 'pb 6/9/2013 16:20'!
time
	^ time! !

Clock subclass: #RealClock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RealClock methodsFor: 'accesing' stamp: 'pb 6/9/2013 16:40'!
time
	^Time now.! !

Object subclass: #NumberSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

NumberSequence subclass: #ConsecutiveNumberSequence
	instanceVariableNames: 'nextValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ConsecutiveNumberSequence methodsFor: 'initialize-release' stamp: 'JuanEdi 6/9/2013 23:31'!
initialize

	nextValue := 1.! !

!ConsecutiveNumberSequence methodsFor: 'value generation' stamp: 'JuanEdi 6/9/2013 23:28'!
nextValue
	| returnValue |
	returnValue := nextValue.
	nextValue := nextValue + 1.
	
	^ returnValue.! !

!NumberSequence methodsFor: 'value generation' stamp: 'JuanEdi 6/9/2013 23:25'!
nextValue
	
	^ self subclassResponsibility.! !

Object subclass: #PluggableStubPaymentTerminal
	instanceVariableNames: 'onDebitClosure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PluggableStubPaymentTerminal methodsFor: 'debit-actions' stamp: 'FranciscoWesner 6/4/2013 17:57'!
debit: anAmount from: aCard 
	
	onDebitClosure value: anAmount value: aCard.! !

!PluggableStubPaymentTerminal methodsFor: 'initialize-release' stamp: 'pb 6/9/2013 11:54'!
initialize

	onDebitClosure := [:x :y | nil ].! !

!PluggableStubPaymentTerminal methodsFor: 'debit-actions' stamp: 'JuanEdi 6/3/2013 21:04'!
onDebitDo: aBlock

	onDebitClosure := aBlock.! !

Object subclass: #RestInterface
	instanceVariableNames: 'userRegistry catalog clock cartsSessions cashier cartIdsSequence transactionIdsSequence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RestInterface class methodsFor: 'error-messages' stamp: 'FW 6/8/2013 17:52'!
inactivityTimeoutMessage
	^ 'Trying to operate with a cart after 30 minutes of inactivity'! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/5/2013 20:46'!
invalidAuthenticationData
	^ 'Invalid Authentication data'! !

!RestInterface class methodsFor: 'instance-creation' stamp: 'pb 6/9/2013 17:21'!
newUsingCartSessions: aCartSessions userRegistry: aUserRegistry catalog: aCatalog cashier: aCashier andClock: aClock.
	
	^ self new withCartsSessions: aCartSessions userRegistry: aUserRegistry catalog: aCatalog cashier: aCashier andClock: aClock.! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/8/2013 12:18'!
unknownBookMessage
	^ 'The book does not exist.'! !

!RestInterface class methodsFor: 'error-messages' stamp: 'JuanEdi 6/8/2013 11:57'!
unknownCartMessage
	^ 'Trying to operate with unknown cart'! !

!RestInterface methodsFor: 'adding' stamp: 'JuanEdi 6/9/2013 22:39'!
addToCart: aCartId isbn: aBookISBN copies: numberOfCopies withUser: aUserId andPassword: aPassword

	self validateCartExists: aCartId.
	self validateBookExists: aBookISBN.
	self validateUserExists: aUserId.
	self authenticateUser: aUserId withPassword: aPassword.
	
	(self cartForId: aCartId) add: aBookISBN withOccurrences: numberOfCopies.! !

!RestInterface methodsFor: 'validating' stamp: 'pb 6/9/2013 15:33'!
authenticateUser: aUserId withPassword: aPassword
	((userRegistry at: aUserId) = aPassword) ifFalse:
		[AuthenticationFailed signal: RestInterface invalidAuthenticationData]! !

!RestInterface methodsFor: 'carts-handling' stamp: 'JuanEdi 6/9/2013 22:58'!
cartForId: aCartId

	[ ^ (cartsSessions at: aCartId) cart ]
		on: SessionExpiredError
		do: [
			cartsSessions removeKey: aCartId.
			self error: RestInterface unknownCartMessage
		]! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:05'!
checkoutCart: aCartId for: aUserId withPassword: aPassword usingCard: aCardNumber expiringOn: aMonth andOwner: aUserName
	| card |
	self validateCartExists: aCartId.
	self validateUserExists: aUserId.
	self authenticateUser: aUserId withPassword: aPassword.
	card := Card ownedBy: aUserName withNumber: aCardNumber expiring: aMonth.
	cashier checkout: (cartsSessions at: aCartId) cart forCustomer: 'Pedro' usingCard: card.
	^ self newTransactionId! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'pb 6/9/2013 18:08'!
listCartWithId: aCartId usingUser: userId andPassword: userPassword 
	| theCart |
	self validateUserExists: userId.
	self authenticateUser: userId withPassword: userPassword.
	
	self validateCartExists: aCartId.

	theCart := (cartsSessions at: aCartId) cart.
	^theCart elements.! !

!RestInterface methodsFor: 'carts-handling' stamp: 'JuanEdi 6/9/2013 22:57'!
newCartForUserWithId: aUserId andPassword: aPassword 

	| cartId cart session |

	self validateUserExists: aUserId.
	self authenticateUser: aUserId withPassword: aPassword.
	
	cartId := self newCartId.
	cart := ShoppingCart newWithCatalog: catalog.
	session := Session newForUser: aUserId cart: cart usingClock: clock expiringIn: self timeUntilSessionExpires.
	
	cartsSessions at: cartId put: session.

	^ cartId.! !

!RestInterface methodsFor: 'carts-handling' stamp: 'JuanEdi 6/9/2013 23:36'!
newCartId
	
	^ cartIdsSequence nextValue.! !

!RestInterface methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 23:42'!
newTransactionId
	
	^ transactionIdsSequence nextValue.! !

!RestInterface methodsFor: 'carts-handling' stamp: 'JuanEdi 6/9/2013 22:36'!
timeUntilSessionExpires

	^ 30 minutes.! !

!RestInterface methodsFor: 'validating' stamp: 'pb 6/9/2013 17:18'!
validateBookExists: aBookISBN

	(catalog includesKey: aBookISBN)
		ifFalse: [ self error: RestInterface unknownBookMessage ]! !

!RestInterface methodsFor: 'validating' stamp: 'pb 6/9/2013 15:55'!
validateCartExists: aCartId

	(cartsSessions includesKey: aCartId)
		ifFalse: [ self error: RestInterface unknownCartMessage ]! !

!RestInterface methodsFor: 'validating' stamp: 'JuanEdi 6/5/2013 21:01'!
validateUserExists: aUserId
	(userRegistry keys includes: aUserId) ifFalse:
		[AuthenticationFailed signal: RestInterface invalidAuthenticationData]! !

!RestInterface methodsFor: 'instance-creation' stamp: 'JuanEdi 6/9/2013 23:35'!
withCartsSessions: aCartSessionsList userRegistry: aUserRegistry catalog: aCatalog cashier: aCashier andClock: aClock
	
	cartsSessions := aCartSessionsList.
	userRegistry := aUserRegistry.
	catalog := aCatalog.
	cashier := aCashier.
	clock := aClock.
	cartIdsSequence := ConsecutiveNumberSequence new.
	transactionIdsSequence := ConsecutiveNumberSequence new.! !

Object subclass: #Session
	instanceVariableNames: 'userId cart lastUsage clock timeUntilExpiration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Session class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:09'!
expiredSessionMessage
	
	^ 'Shopping session has expired.'! !

!Session class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:11'!
newForUser: aUserId cart: aShoppingCart usingClock: aClock expiringIn: aDuration
	
	^ self new initializeWithUser: aUserId cart: aShoppingCart usingClock: aClock expiringIn: aDuration.! !

!Session methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:24'!
cart
	self validateSessionIsNotExpired.
	self updateLastUsage.
	
	^ cart! !

!Session methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 21:59'!
initializeWithUser: aUserId cart: aShoppingCart usingClock: aClock 
	
	userId := aUserId.
	cart := aShoppingCart.
	clock := aClock.
	lastUsage := aClock time.! !

!Session methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:12'!
initializeWithUser: aUserId cart: aShoppingCart usingClock: aClock expiringIn: aDuration
	
	userId := aUserId.
	cart := aShoppingCart.
	clock := aClock.
	lastUsage := aClock time.
	timeUntilExpiration := aDuration.! !

!Session methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:24'!
updateLastUsage

	lastUsage := clock time.! !

!Session methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/9/2013 22:58'!
validateSessionIsNotExpired
	
	((clock time - lastUsage) >= timeUntilExpiration)
		ifTrue: [ self error: SessionExpiredError signal. ].
	
	
	^ cart! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:23'!
invalidAmountOfProductsMessage
	
	^ 'Amount of products to add should be greater than zero'.! !

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'error-messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/8/2013 12:23'!
add: aProduct 
	self add: aProduct withOccurrences: 1.! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/8/2013 12:23'!
add: aProduct withOccurrences: aNumber 
	
	self validatePossitiveAmountOfProducts: aNumber.
	self validateCatalogContainsProduct: aProduct.
		
	products add: aProduct withOccurrences: aNumber.! !

!ShoppingCart methodsFor: 'enumerating' stamp: 'JuanEdi 6/1/2013 22:54'!
do: aBlock
	"Itera sobre cada ocurrencia individual de cada producto del carro."

	products do: aBlock! !

!ShoppingCart methodsFor: 'as yet unclassified' stamp: 'pb 6/9/2013 17:01'!
elements
	^ products.! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 6/1/2013 22:24'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Bag new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'accessing' stamp: 'JuanEdi 6/1/2013 22:31'!
itemCount
	"Retorna la cantidad de items en total, contando todas las unidades de cada item agregado."
	^ products size.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 6/1/2013 22:55'!
occurrencesOf: aProduct 
	
	^ products occurrencesOf: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 6/1/2013 22:34'!
validateCatalogContainsProduct: aProduct

	(catalog keys includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:22'!
validatePossitiveAmountOfProducts: anAmount

	(anAmount > 0) ifFalse: [ self error: ShoppingCart invalidAmountOfProductsMessage]! !

Object subclass: #Ticket
	instanceVariableNames: 'customer entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Ticket class methodsFor: 'instance-creation' stamp: 'JuanEdi 6/10/2013 01:14'!
newForUser: aUser withPurchases: aBagOfItems usingCatalog: aCatalog
	^self new initializeWithUser: aUser purchases: aBagOfItems usingCatalog: aCatalog.! !

!Ticket class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:13'!
noPurchasesMessage
	
	^ 'Cannot create a ticket with no purchases'! !

!Ticket class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:21'!
unknownProductMessage
	
	^ 'Trying to purchase a product not present in the catalog.'! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:16'!
customer
	
	^ customer! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:37'!
entries
	^ entries.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:16'!
initializeWithUser: aCustomer purchases: aBagOfItems usingCatalog: aCatalog 
	
	self validateAtLeastOnePurchase: aBagOfItems.
	self validateAllItemsIn: aBagOfItems arePresentIn: aCatalog.
	
	customer := aCustomer.
	entries := Set new.
	aBagOfItems valuesAndCounts keysAndValuesDo: [ :anItem :itemCount |
		entries add: (TicketEntry withCount: itemCount ofItem: anItem withTotalAmount: itemCount * (aCatalog at: anItem) )
	]! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:14'!
totalAmount

	^ (entries collect: [ :anEntry | anEntry totalAmount ]) inject: 0 into: [ :anAmount :total | anAmount + total ].! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:25'!
validateAllItemsIn: aBag arePresentIn: aCatalog 
	
	(aBag allSatisfy: [ :anItem | aCatalog includesKey: anItem ])
		ifFalse: [ self error: Ticket unknownProductMessage ]! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:18'!
validateAtLeastOnePurchase: aBagOfItems
	
	(aBagOfItems isEmpty)
		ifTrue: [ self error: Ticket noPurchasesMessage ]! !

Object subclass: #TicketEntry
	instanceVariableNames: 'itemCount itemDescription totalAmount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TicketEntry class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:35'!
withCount: aNumber ofItem: anItem withTotalAmount: anAmount 
	
	^ self new withCount: aNumber ofItem: anItem withTotalAmount: anAmount.! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:44'!
= anotherObject
	^ (anotherObject isKindOf: TicketEntry)
		and: [
			(self itemCount = anotherObject itemCount) 
			and: ((self itemDescription = anotherObject itemDescription)
			and: (self totalAmount = anotherObject totalAmount))
		]! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:09'!
hash

	^ (self printString) hash.! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:43'!
itemCount
	^ itemCount.! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:44'!
itemDescription
	
	^ itemDescription! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 02:10'!
printString

	^ (itemCount printString) , ' ' , itemDescription , ' ' , (totalAmount printString).! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:44'!
totalAmount
	
	^ totalAmount! !

!TicketEntry methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/10/2013 01:53'!
withCount: aNumber ofItem: anItem withTotalAmount: anAmount 
	
	itemCount := aNumber.
	itemDescription := anItem.
	totalAmount := anAmount.! !

Error subclass: #AuthenticationFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Error subclass: #CardWithNoCreditError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

Error subclass: #SessionExpiredError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!
